@@ -1,5 +1,8 @@
+### R code from vignette source 'Rnw/algo_farrington.Rnw'
+### Encoding: ISO8859-1
+
 ###################################################
-### chunk number 1: 
+### code chunk number 1: algo_farrington.Rnw:25-35
 ###################################################
 anscombe.residuals <- function(m,phi) {
   y <- m$y
@@ -13,23 +16,23 @@ anscombe.residuals <- function(m,phi) {
 }
 
 
-###################################################
-### chunk number 2: 
-###################################################
-algo.farrington.assign.weights <- function(s) {
-  #s_i^(-2) for s_i<1 and 1 otherwise
-  gamma <- length(s)/(sum(  (s^(-2))^(s>1) ))
-  omega <- numeric(length(s)) 
-  omega[s>1] <- gamma*(s[s>1]^(-2))
-  omega[s<=1] <- gamma
-  return(omega)
+################################################################################
+# WEIGHTS FUNCTION
+################################################################################
+
+algo.farrington.assign.weights <- function(s,weightsThreshold=1) {
+    #s_i^(-2) for s_i<weightsThreshold and 1 otherwise
+    gamma <- length(s)/(sum(    (s^(-2))^(s>weightsThreshold) ))
+    omega <- numeric(length(s))
+    omega[s>weightsThreshold] <- gamma*(s[s>weightsThreshold]^(-2))
+    omega[s<=weightsThreshold] <- gamma
+    return(omega)
 }
 
-
 ###################################################
-### chunk number 3: 
+### code chunk number 3: algo_farrington.Rnw:136-305
 ###################################################
-algo.farrington.fitGLM <- function(response,wtime,timeTrend=TRUE,reweight=TRUE) {
+algo.farrington.fitGLM <- function(response,wtime,timeTrend=TRUE,reweight=TRUE,...) {
   #Model formula depends on whether to include a time trend or not.
   theModel <- as.formula(ifelse(timeTrend, "response~1+wtime","response~1"))
 
@@ -40,8 +43,10 @@ algo.farrington.fitGLM <- function(response,wtime,timeTrend=TRUE,reweight=TRUE)
   if (!model$converged) {
     #Try without time dependence
     if (timeTrend) {
-     model <- glm(response ~ 1, family = quasipoisson(link="log"))
-     cat("Warning: No convergence with timeTrend -- trying without.\n")
+      cat("Warning: No convergence with timeTrend -- trying without.\n")
+      #Set model to one without time trend
+      theModel <- as.formula("response~1")
+      model <- glm(response ~ 1, family = quasipoisson(link="log"))
     } 
 
     if (!model$converged) {
@@ -85,14 +90,14 @@ algo.farrington.fitGLM <- function(response,wtime,timeTrend=TRUE,reweight=TRUE)
 # necessary ones for the algo.farrington procedure work.
 ######################################################################
 
-algo.farrington.fitGLM.fast <- function(response,wtime,timeTrend=TRUE,reweight=TRUE) {
+algo.farrington.fitGLM.fast <- function(response,wtime,timeTrend=TRUE,reweight=TRUE, ...) {
   #Create design matrix and formula needed for the terms object
   #Results depends on whether to include a time trend or not.
   if (timeTrend) {
-    design<-cbind(intercept=1,wtime) 
+    design<-cbind(intercept=1,wtime=wtime) 
     Formula<-response~wtime 
   } else {
-    design<-matrix(1,nrow=length(wtime))
+    design<-matrix(1,nrow=length(wtime),dimnames=list(NULL,c("intercept")))
     Formula<-response~1
   }
   
@@ -103,10 +108,14 @@ algo.farrington.fitGLM.fast <- function(response,wtime,timeTrend=TRUE,reweight=T
    if (!model$converged) {
       #Try without time dependence
      if (timeTrend) {
-       model <- glm.fit(design[,1,drop=FALSE],response, family = quasipoisson(link = "log"))
-       Formula<-response~1
        cat("Warning: No convergence with timeTrend -- trying without.\n")
+       #Drop time from design matrix
+       design <- design[,1,drop=FALSE]
+       #Refit
+       model <- glm.fit(design,response, family = quasipoisson(link = "log"))
+       Formula<-response~1
      } 
+     #No convergence and no time trend. That's not good.
    }
 
    #Fix class of output to glm/lm object in order for anscombe.residuals to work
@@ -129,17 +138,75 @@ algo.farrington.fitGLM.fast <- function(response,wtime,timeTrend=TRUE,reweight=T
    model$phi <- phi
    model$wtime <- wtime
    model$response <- response
-   model$terms<-terms(Formula)
+   model$terms <- terms(Formula)
    # cheating a bit, all methods for glm may not work
-   class(model)<-c("algo.farrington.glm","glm") 
+   class(model)<-c("algo.farrington.glm","glm","lm") # 23/10/2012 (SM):
+                                        # added "lm" class to avoid warnings
+                                        # from predict.lm about fake object
    #Done
   return(model)
 }
 
+######################################################################
+# Experimental function to include a population offset in the 
+# farrington procedure based on algo.farrington.fitGLM
+# Alternative: include populationOffset argument in the two other
+# fit functions, but I suspect use of this is not so common
+#
+# Parameters:
+#  takes an additional "population" parameter
+######################################################################
+
+algo.farrington.fitGLM.populationOffset <- function(response,wtime,population,timeTrend=TRUE,reweight=TRUE,...) {
+  #Model formula depends on whether to include a time trend or not.
+  theModel <- as.formula(ifelse(timeTrend, "response~offset(log(population)) + 1 + wtime","response~offset(log(population)) + 1"))
+
+  #Fit it -- this is slow. An improvement would be to use glm.fit here.
+  model <- glm(theModel, family = quasipoisson(link="log"))
+    
+ #Check convergence - if no convergence we return empty handed.
+  if (!model$converged) {
+    #Try without time dependence
+    if (timeTrend) {
+     model <- glm(response ~ 1, family = quasipoisson(link="log"))
+     cat("Warning: No convergence with timeTrend -- trying without.\n")
+    } 
+
+    if (!model$converged) {
+      cat("Warning: No convergence in this case.\n")
+      print(cbind(response,wtime))
+      return(NULL)
+    }
+  }
+
+  #Overdispersion parameter phi
+  phi <- max(summary(model)$dispersion,1)
+  
+  #In case reweighting using Anscome residuals is requested
+  if (reweight) {
+    s <- anscombe.residuals(model,phi)
+    omega <- algo.farrington.assign.weights(s)
+    model <- glm(theModel,family=quasipoisson(link="log"),weights=omega)
+    #Here, the overdispersion often becomes small, so we use the max
+    #to ensure we don't operate with quantities less than 1.
+    phi <- max(summary(model)$dispersion,1)
+  } # end of refit.
+  
+
+  #Add wtime, response and phi to the model
+  model$phi <- phi
+  model$wtime <- wtime
+  model$response <- response
+  model$population <- population
+  #Done
+  return(model)
+}
+
+
 
 
 ###################################################
-### chunk number 4: 
+### code chunk number 4: algo_farrington.Rnw:344-370
 ###################################################
 
 algo.farrington.threshold <- function(pred,phi,alpha=0.01,skewness.transform="none",y) {
@@ -170,7 +237,7 @@ algo.farrington.threshold <- function(pred,phi,alpha=0.01,skewness.transform="no
 
 
 ###################################################
-### chunk number 5: 
+### code chunk number 5: algo_farrington.Rnw:412-451
 ###################################################
 ######################################################################
 # Compute indices of reference value using Date class
@@ -192,19 +259,29 @@ algo.farrington.threshold <- function(pred,phi,alpha=0.01,skewness.transform="no
 refvalIdxByDate <- function(t0, b, w, epochStr, epochs) {
   refDays <- NULL
   refPoints <- seq( t0, length=b+1, by="-1 year")[-1]
+
+  #Loop over all b-lagged points and append appropriate w-lagged points
   for (j in 1:length(refPoints)) {
-    refPointWindow <- c(rev(seq(refPoints[j], length=w+1, by=paste("-",epochStr,sep=""))),
-                        seq(refPoints[j], length=w+1, by=epochStr)[-1])
-    refDays <- append(refDays,refPointWindow)
+      refPointWindow <- c(rev(seq(refPoints[j], length=w+1, by=paste("-",epochStr,sep=""))),
+                          seq(refPoints[j], length=w+1, by=epochStr)[-1])
+      refDays <- append(refDays,refPointWindow)
   }
   if (epochStr == "1 week") {
-    #By convention: always go back to the closest monday 
-    #(but always back, never ahead!)
-    refDays <- refDays -  (as.numeric(format(refDays, "%w")) - 1)
+      #What weekday is t0 (0=Sunday, 1=Monday, ...)
+      epochWeekDay <- as.numeric(format(t0,"%w"))
+      #How many days to go forward to obtain the next "epochWeekDay", i.e. (d0 - d) mod 7
+      dx.forward <- (epochWeekDay - as.numeric(format(refDays,"%w"))) %% 7
+      #How many days to go backward to obtain the next "epochWeekDay", i.e. (d - d0) mod 7
+      dx.backward <- (as.numeric(format(refDays,"%w")) - epochWeekDay) %% 7
+      #What is shorter - go forward or go backward? 
+      #By convention: always go to the closest weekday as t0
+      refDays <- refDays + ifelse(dx.forward < dx.backward, dx.forward, -dx.backward)
   }
   if (epochStr == "1 month") {
-    #By convention: go back in time to  closest 1st of month
-    refDays <- refDays -  (as.numeric(format(refDays, "%d")) - 1)
+      #What day of the month is t0 (it is assumed that all epochs have the same value here)
+      epochDay <- as.numeric(format(t0,"%d"))
+      #By convention: go back in time to closest 1st of month
+      refDays <- refDays -  (as.numeric(format(refDays, "%d")) - epochDay)
   }
   #Find the index of these reference values
   wtime <- match(as.numeric(refDays), epochs)
@@ -214,14 +291,20 @@ refvalIdxByDate <- function(t0, b, w, epochStr, epochs) {
 
 
 ###################################################
-### chunk number 6: 
+### code chunk number 6: algo_farrington.Rnw:571-769
 ###################################################
-algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, reweight=TRUE, verbose=FALSE,alpha=0.01,trend=TRUE,limit54=c(5,4),powertrans="2/3",fitFun=c("algo.farrington.fitGLM.fast","algo.farrington.fitGLM"))) { 
+
+algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, reweight=TRUE, verbose=FALSE,alpha=0.01,trend=TRUE,limit54=c(5,4),powertrans="2/3",fitFun=c("algo.farrington.fitGLM.fast","algo.farrington.fitGLM","algo.farrington.fitGLM.populationOffset"))) { 
   #Fetch observed
   observed <- disProgObj$observed
   freq <- disProgObj$freq
   epochStr <- switch( as.character(freq), "12" = "1 month","52" =  "1 week","365" = "1 day")
-
+  #Fetch population (if it exists)
+  if (!is.null(disProgObj$populationFrac)) {
+    population <- disProgObj$populationFrac } 
+  else { 
+    population <- rep(1,length(observed))
+  }
 
   ######################################################################
   # Fix missing control options
@@ -241,7 +324,7 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
   if (is.null(control$fitFun))   {
     control$fitFun="algo.farrington.fitGLM.fast"
   } else {
-    control$fitFun <- match.arg(control$fitFun, c("algo.farrington.fitGLM.fast","algo.farrington.fitGLM"))
+    control$fitFun <- match.arg(control$fitFun, c("algo.farrington.fitGLM.fast","algo.farrington.fitGLM","algo.farrington.fitGLM.populationOffset"))
   }
 
   #Use special Date class mechanism to find reference months/weeks/days
@@ -255,7 +338,11 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
   if (!((control$limit54[1] >= 0) &  (control$limit54[2] > 0))) {
     stop("The limit54 arguments are out of bounds: cases >= 0 and period > 0.")
   }
-
+  #Check control$range is within bounds.
+  if (any((control$range < 1) | (control$range > length(disProgObj$observed)))) {
+      stop("Range values are out of bounds (has to be within 1..",length(disProgObj$observed)," for the present data).")
+  }
+  
   # initialize the necessary vectors
   alarm <- matrix(data = 0, nrow = length(control$range), ncol = 1)
   trend <- matrix(data = 0, nrow = length(control$range), ncol = 1)
@@ -274,7 +361,7 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
     #shortObserved <- observed[1:(maxRange - k + 1)]
 
     if (control$verbose) { cat("k=",k,"\n")}
-
+            
     #Find index of all epochs, which are to be used as reference values
     #i.e. with index k-w,..,k+w 
     #in the years (current year)-1,...,(current year)-b
@@ -283,18 +370,25 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
       for (i in control$b:1){
         wtimeAll <- append(wtimeAll,seq(k-freq*i-control$w,k-freq*i+control$w,by=1))
       }
-      #Select them as reference values
+      #Select them as reference values - but only those who exist
       wtime <- wtimeAll[wtimeAll>0]
       if (length(wtimeAll) != length(wtime)) {
-        warning("Some reference values did not exist (index<1).")
+          warning("@ range= ",k,": With current b and w then ",length(wtimeAll) - length(wtime),"/",length(wtimeAll), " reference values did not exist (index<1).")
       }
     } else { #Alternative approach using Dates
       t0 <- as.Date(disProgObj$week[k], origin="1970-01-01")
-      wtime <- refvalIdxByDate( t0=t0, b=control$b, w=control$w, epochStr=epochStr, epochs=disProgObj$week)
+      wtimeAll <- refvalIdxByDate( t0=t0, b=control$b, w=control$w, epochStr=epochStr, epochs=disProgObj$week)
+      #Select them as reference values (but only those not being NA!)
+      wtime <- wtimeAll[!is.na(wtimeAll)]
+      #Throw warning if necessary
+      if (length(wtimeAll) != length(wtime)) {
+          warning("@ range= ",k,": With current b and w then ",length(wtimeAll) - length(wtime),"/",length(wtimeAll), " reference values did not exist (index<1).")
+      }
     }
     
     #Extract values from indices
     response <- observed[wtime]
+    pop <- population[wtime]
 
     if (control$verbose) { print(response)}
 
@@ -302,8 +396,7 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
     #Fit the model with overdispersion -- the initial fit
     ######################################################################
     #New feature: fitFun can now be the fast function for fitting the GLM
-    model <- do.call(control$fitFun, args=list(response=response,wtime=wtime,timeTrend=control$trend,reweight=control$reweight))
-#    model <- algo.farrington.fitGLM.fast(control$fitFun, response=response,wtime=wtime,timeTrend=control$trend,reweight=control$reweight)
+    model <- do.call(control$fitFun, args=list(response=response,wtime=wtime,population=pop,timeTrend=control$trend,reweight=control$reweight))
 
     #Stupid check to pass on NULL values from the algo.farrington.fitGLM proc.
     if (is.null(model)) return(model)
@@ -316,12 +409,14 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
     #2) the predicted value is not larger than any observed value
     #3) the historical data span at least 3 years.
     doTrend <- control$trend
-    if (control$trend) {
+#Bug discovered by Julia Kammerer and Sabrina Heckl: Only investigate trend if it actually was part of the GLM
+    #if (control$trend) {
+    if ("wtime" %in% names(coef(model))){
       #is the p-value for the trend significant (0.05) level
       p <- summary.glm(model)$coefficients["wtime",4]
       significant <- (p < 0.05)
       #prediction for time k
-      mu0Hat <- predict.glm(model,data.frame(wtime=c(k)),type="response")
+      mu0Hat <- predict.glm(model,data.frame(wtime=c(k),population=population[k]),type="response")
       #have to use at least three years of data to allow for a trend
       atLeastThreeYears <- (control$b>=3)
       #no horrible predictions
@@ -331,8 +426,7 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
       #it is removed. Only necessary to check this if a trend is requested.
       if (!(atLeastThreeYears && significant && noExtrapolation)) {
         doTrend <- FALSE
-        model <- do.call(control$fitFun, args=list(response=response,wtime=wtime,timeTrend=FALSE,reweight=control$reweight))
-#        model <- algo.farrington.fitGLM.fast(control$fitFun, response=response,wtime=wtime,timeTrend=FALSE,reweight=control$reweight)
+        model <- do.call(control$fitFun, args=list(response=response,wtime=wtime,population=pop,timeTrend=FALSE,reweight=control$reweight))
       }
     } else {
       doTrend <- FALSE
@@ -345,7 +439,7 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
     ######################################################################
     #Predict value - note that the se is the mean CI
     #and not the prediction error of a single observation
-    pred <- predict.glm(model,data.frame(wtime=c(k)),dispersion=model$phi,
+    pred <- predict.glm(model,data.frame(wtime=c(k),population=population[k]),dispersion=model$phi,
                         type="response",se.fit=TRUE)
     #Calculate lower and upper threshold
     lu <- algo.farrington.threshold(pred,model$phi,skewness.transform=control$powertrans,alpha=control$alpha, observed[k])
